"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.GsOTP = exports.isGsOTPError = void 0;
const got_1 = require("got");
const jsonBigInt_1 = require("./jsonBigInt");
const GSOTP_HOST = 'https://api.gsotp.com';
/**
 * Check whether an error is a gsOTP standard error or not<br>
 * These objects have two fields: `code` and `message`
 * @param error The error object to check
 * @returns `error` is a standard gsOTP error
 */
function isGsOTPError(error) {
    return !!(error && error.code && error.message && error.constructor === Object);
}
exports.isGsOTPError = isGsOTPError;
class GsOTP {
    /**
     * @param apiKey Your API key in gsOTP.com.
     * @param manual Generate OTP code automatically or you gonna to enter it manually. <br>Default is `false`.
     * @param language Target language for showing error messages.
     */
    constructor(apiKey, manual = false, language = 'fa-IR') {
        this.apiKey = apiKey;
        this.language = language;
        this.sendSMS = this.sendSMS.bind(this);
        this.sendIVR = this.sendIVR.bind(this);
        this.sendGapMessage = this.sendGapMessage.bind(this);
        this.verify = this.verify.bind(this);
        this.getStatus = this.getStatus.bind(this);
    }
    request(path, body) {
        let language = this.language;
        if ('language' in body) {
            language = body.language;
            delete body.language;
        }
        return got_1.default(GSOTP_HOST + path, {
            headers: {
                'accept-language': language,
                'content-type': 'application/json',
                apiKey: this.apiKey
            },
            method: 'post',
            responseType: 'text',
            body: jsonBigInt_1.stringify(body),
        })
            .then(response => {
            let result;
            try {
                result = jsonBigInt_1.parse(response.body);
            }
            catch (err) {
                return Promise.reject(err);
            }
            if (result.error) {
                return Promise.reject(result);
            }
            delete result.status;
            delete result.error;
            return result;
        })
            .catch((error) => {
            if (error.response) {
                try {
                    error = jsonBigInt_1.parse(error.response.body);
                }
                catch (error) { }
            }
            if (error.status === 'error' && isGsOTPError(error.error)) {
                return Promise.reject(error.error);
            }
            return Promise.reject(error);
        });
    }
    send(method, options) {
        const body = Object.assign(Object.assign({ method, smart: false }, options), { expireTime: options.expireTime ? +options.expireTime : undefined });
        return this.request('/otp/send', body)
            .then(result => result.referenceID);
    }
    /**
     * Send OTP Code via SMS
     * @param options Send options
     * @returns Reference ID
     */
    sendSMS(options) {
        return this.send('sms', options);
    }
    /**
     * Send OTP Code via IVR (Interactive voice response)
     * @param options Send options
     * @returns Reference ID
     */
    sendIVR(options) {
        return this.send('ivr', options);
    }
    /**
     * Send OTP Code via Gap Messenger ([gap.im](https://gap.im))
     * @returns Reference ID
     */
    sendGapMessage(options) {
        return this.send('gap', options);
    }
    /**
     * Verify the user entered code.
     * @returns If the code is correct the promise resolves.
     */
    verify(options) {
        return this.request('/otp/verify', options)
            .then(() => { });
    }
    /**
     * Get status of the sent OTP.
     * @param referenceID
     */
    getStatus(options) {
        return this.request('/otp/status', options);
    }
}
exports.GsOTP = GsOTP;
